package com.link.core.mybatis;

import com.link.core.cllog.LCLogger;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.apache.ibatis.binding.MapperRegistry;
import org.apache.ibatis.builder.xml.XMLMapperEntityResolver;
import org.apache.ibatis.executor.ErrorContext;
import org.apache.ibatis.io.Resources;
import org.apache.ibatis.parsing.XNode;
import org.apache.ibatis.parsing.XPathParser;
import org.apache.ibatis.session.Configuration;
import org.springframework.core.io.Resource;

public class XMLMapperUnloader {
   private Configuration configuration;
   private Set<?> loadedResources;
   private Map<?, ?> knownMappers;
   private Collection<String> cacheNames;
   private Collection<String> mappedStatementNames;
   private Collection<String> parameterMapNames;
   private Collection<String> resultMapNames;
   private Collection<String> sqlFragmentNames;
   private Collection<String> keyGeneratorNames;

   public XMLMapperUnloader(Configuration configuration) {
      this.configuration = configuration;
      this.loadedResources = this.getSetField(Configuration.class, configuration, "loadedResources");
      this.knownMappers = this.getMapField(MapperRegistry.class, configuration.getMapperRegistry(), "knownMappers");
      this.cacheNames = configuration.getCacheNames();
      this.mappedStatementNames = configuration.getMappedStatementNames();
      this.parameterMapNames = configuration.getParameterMapNames();
      this.resultMapNames = configuration.getResultMapNames();
      this.keyGeneratorNames = configuration.getKeyGeneratorNames();
      this.sqlFragmentNames = configuration.getSqlFragments().keySet();
   }

   public void unloadXml(Resource resource) {
      try {
         XPathParser xPathParser = new XPathParser(resource.getInputStream(), true, this.configuration.getVariables(), new XMLMapperEntityResolver());
         XNode context = xPathParser.evalNode("/mapper");
         String namespace = context.getStringAttribute("namespace");
         this.knownMappers.remove(Resources.classForName(namespace));
         this.loadedResources.remove(resource.toString());
         this.cacheNames.remove(namespace);
         this.cleanMappedStatements(context.evalNodes("select|insert|update|delete"), namespace);
         this.cleanParameterMaps(context.evalNodes("/mapper/parameterMap"), namespace);
         this.cleanResultMaps(context.evalNodes("/mapper/resultMap"), namespace);
         this.cleanKeyGenerators(context.evalNodes("insert|update"), namespace);
         this.cleanSqlElements(context.evalNodes("/mapper/sql"), namespace);
      } catch (Exception var8) {
         Exception e = var8;
         LCLogger.withException(e);
      } finally {
         ErrorContext.instance().reset();
      }

   }

   private Set<?> getSetField(Class<?> clazz, Object object, String fieldName) {
      try {
         Field setField = clazz.getDeclaredField(fieldName);
         setField.setAccessible(true);
         return (Set)setField.get(object);
      } catch (Exception var5) {
         Exception e = var5;
         throw new RuntimeException(e.getMessage(), e);
      }
   }

   private Map<?, ?> getMapField(Class<?> clazz, Object object, String fieldName) {
      try {
         Field setField = clazz.getDeclaredField(fieldName);
         setField.setAccessible(true);
         return (Map)setField.get(object);
      } catch (Exception var5) {
         Exception e = var5;
         throw new RuntimeException(e.getMessage(), e);
      }
   }

   private void cleanMappedStatements(List<XNode> list, String namespace) {
      Iterator var3 = list.iterator();

      while(var3.hasNext()) {
         XNode node = (XNode)var3.next();
         String id = node.getStringAttribute("id");
         this.mappedStatementNames.remove(namespace + "." + id);
      }

   }

   private void cleanParameterMaps(List<XNode> list, String namespace) {
      Iterator var3 = list.iterator();

      while(var3.hasNext()) {
         XNode node = (XNode)var3.next();
         String id = node.getStringAttribute("id");
         this.parameterMapNames.remove(namespace + "." + id);
      }

   }

   private void cleanResultMaps(List<XNode> list, String namespace) {
      Iterator var3 = list.iterator();

      while(var3.hasNext()) {
         XNode node = (XNode)var3.next();
         String id = node.getStringAttribute("id", node.getValueBasedIdentifier());
         this.resultMapNames.remove(id);
         this.resultMapNames.remove(namespace + "." + id);
         this.clearResultMaps(node, namespace);
      }

   }

   private void clearResultMaps(XNode xNode, String namespace) {
      Iterator var3 = xNode.getChildren().iterator();

      while(true) {
         XNode node;
         do {
            if (!var3.hasNext()) {
               return;
            }

            node = (XNode)var3.next();
         } while(!"association".equals(node.getName()) && !"collection".equals(node.getName()) && !"case".equals(node.getName()));

         if (node.getStringAttribute("select") == null) {
            this.resultMapNames.remove(node.getStringAttribute("id", node.getValueBasedIdentifier()));
            this.resultMapNames.remove(namespace + "." + node.getStringAttribute("id", node.getValueBasedIdentifier()));
            if (node.getChildren() != null && !node.getChildren().isEmpty()) {
               this.clearResultMaps(node, namespace);
            }
         }
      }
   }

   private void cleanKeyGenerators(List<XNode> list, String namespace) {
      Iterator var3 = list.iterator();

      while(var3.hasNext()) {
         XNode node = (XNode)var3.next();
         String id = node.getStringAttribute("id");
         this.keyGeneratorNames.remove(id + "!selectKey");
         this.keyGeneratorNames.remove(namespace + "." + id + "!selectKey");
      }

   }

   private void cleanSqlElements(List<XNode> list, String namespace) {
      Iterator var3 = list.iterator();

      while(var3.hasNext()) {
         XNode node = (XNode)var3.next();
         String id = node.getStringAttribute("id");
         this.sqlFragmentNames.remove(id);
         this.sqlFragmentNames.remove(namespace + "." + id);
      }

   }

   private String getValue(Resource resource) {
      try {
         String contentLength = String.valueOf(resource.contentLength());
         String lastModified = String.valueOf(resource.lastModified());
         return contentLength + lastModified;
      } catch (IOException var4) {
         IOException e = var4;
         throw new RuntimeException(e.getMessage(), e);
      }
   }
}
